function [ y, nfilter, z0 ] = adaptivenotch( x, nfilter )

%%
% Summary:
%   Function implementing adaptive notch filtering (single pole)
%
% Arguments:
%   x       [vector, complex]: input vector containing the samples to filtered
%   nfilter [struct]: structure used to represent the notch filter
%           This struct needs to be updated in order to avoid transients
%           from one epoch to another
%
%           nfilter = struct( 'z0', 0, ...              % filter zero
%                             'xi', 0, ...              % output of the AR
%                                                       part
%                             'ka', 0.9, ...            % contraction factor
%                             'mu', 0 )                 % adaptation step
%
% Returns
%   y       [vector, complex]: filtered vector
%   nfilter [struct]: updated notch filter
%   z0      [vector]: evolution of the complex zero
%
% History:
%   May 23/12 - Function created by Daniele Borio.
%%

% First check if the filter has been initialized
if nfilter.mu == 0,
    nfilter.z0 = ( rand(1, 1) - 0.5 ) + 1i * ( rand(1, 1) - 0.5 );  % initialize the zero in a random position
end    
    
% Adapt the algorith step
energy = var( x );
nfilter.mu = 0.25 * (1 - nfilter.ka ) / energy  ;  % This step can be changed to have faster adaptation


%%
% Perform filtering and adaptation
z0 = zeros( size( x ) );
y = zeros( size( x ) );

for ii = 1:length( x ),    
    % output of the autoregressive part
    xi = x( ii ) + nfilter.ka * nfilter.z0 * nfilter.xi;
    
    % ouput of the moving average part
    y( ii ) = xi - nfilter.z0 * nfilter.xi;
        
    % update the zero position
    nfilter.z0 = nfilter.z0 + nfilter.mu * y( ii ) * conj( nfilter.xi );
    nfilter.xi = xi;    % update the filter state
    z0( ii ) = nfilter.z0;
end
