clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Summary: 
%   Script that applies notch filtering to GNSS data corrupted by 
%   interference                                                   
%
% Returns:
%   Generate a binary files containing the real and imaginary parts of the
%   zero of the notch filter. The zero is adaptively adjusted in order to
%   track the interference variations.
%
%   Data storage:
%   int16 - scaling 2^12 - I/Q interleaved
%
%   History:
%   Jan 10/14 - Script created by Daniele Borio
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Open the input file and set the signal parameters                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen('JammerData.bin','r');

fs = 10e6;       % Sampling frequency
fif = 0e6;    % Intermediate frequency
IsComplex = true;  % Tell if the samples are in I/Q format
Is16Bits  = false;  % Tell if the samples are stored on 16 bits/byte

% Eventually skip some seconds of data
secondToSkip = 0;
bytesToSkip = secondToSkip * fs;

if IsComplex,
    bytesToSkip = 2 * bytesToSkip;
end

if Is16Bits,
    bytesToSkip = 2 * bytesToSkip;
end
    
fseek(fid, bytesToSkip, 'bof');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Processing epochs: number and duration                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Tc = 0.01;      % 10 ms;
blksize = round( Tc * fs );
numEpochs = 1; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Notch filter for interference removal                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Create the notch filter structure
if ~IsComplex,
    error('Real sampling not supported');
end

nfilter = struct( 'z0', 0, ...              % filter zero
                  'xi', 0, ...              % output of the AR part  
                  'ka', 0.8, ...            % contraction factor
                  'mu', 0 );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output files containing the zero position and the filtered samples      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ofid = fopen( 'output.bin', 'wb');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      main processing loop                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ii = 1:numEpochs,

    if rem(ii, 100) == 0,
        ii          % display some info on the processing status
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                           Read the data                             %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if( IsComplex ),
        if( Is16Bits ),
            [rawdata, cnt_data] = fread(fid, 2 * blksize, 'int16');
        else
            [rawdata, cnt_data] = fread(fid, 2 * blksize, 'int8');
        end
        rawdata = rawdata(1:2:end) + 1i * rawdata(2:2:end);
    else
        if( Is16Bits ),
            [rawdata, cnt_data] = fread(fid, blksize, 'int16');
        else
            [rawdata, cnt_data] = fread(fid, blksize, 'int8');
        end
    end
    rawdata = rawdata.';
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                     Perform notch filtering                         %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [ filtdata, nfilter, z0 ] = adaptivenotch( rawdata, nfilter );
    
    if Is16Bits,
        nbits = 16;
    else
        nbits = 8;
    end
    
    % Use the filtered samples only if the interference is clearly detected
    if mean( abs( z0 ) ) > 0.75,
        DumpToFile( filtdata, ofid, 1, nbits );
    else
        DumpToFile( rawdata, ofid, 1, nbits );
    end
end

% close all the file
fclose( fid );
fclose( ofid );

estf = angle( z0 ) / (2 * pi) * fs;
amp = abs( z0 );
time = ( 1:length( z0 ) ) / fs;

[SY, FY, TY ] = spectrogram( rawdata, 32, 20, 1024, fs );
FY = FY - fs/2;
SY = fftshift( SY, 1 );
ax(1) = subplot(2, 1, 1);
imagesc( TY * 1e6, FY / 1e6, real( SY .*conj(SY) ) )
xlim([1 100])
set(gca, 'YDir', 'Normal')
set(gca, 'FontSize', 16)
xlabel('Time [\mu s]')
ylabel('Frequency [MHz]')
title(sprintf('ka = %f', nfilter.ka ))
hold on
plot( time * 1e6, estf / 1e6, 'r--', 'LineWidth', 2)
cLim = get( ax(1), 'CLim' );

ax(2) = subplot(2, 1, 2);
[SF, FF, TF ] = spectrogram( filtdata, 32, 20, 1024, fs );
FF = FF - fs/2;
SF = fftshift( SF, 1 );
imagesc( TF * 1e6, FF / 1e6, real( SF .*conj(SF) ), cLim )
xlim([1 100])
set(gca, 'YDir', 'Normal')
set(gca, 'FontSize', 16)
xlabel('Time [\mu s]')
ylabel('Frequency [MHz]')

linkaxes( ax, 'xy' )

figure
plot( time * 1e6, amp )
set(gca, 'FontSize', 16 )
grid on
axis tight
xlabel('Time [\mu s]')
ylabel('z_0 Amplitude')
xlim([1 100])
ylim([0 1.5])